@@ -1,6 +1,6 @@
 package CGI::Session::Driver::postgresql;
 
-# $Id: postgresql.pm 257 2006-03-16 07:56:41Z markstos $
+# $Id: postgresql.pm 351 2006-11-24 14:16:50Z markstos $
 
 # CGI::Session::Driver::postgresql - PostgreSQL driver for CGI::Session
 #
@@ -15,20 +15,20 @@ use Carp "croak";
 use CGI::Session::Driver::DBI;
 use DBD::Pg qw(PG_BYTEA PG_TEXT);
 
-$CGI::Session::Driver::postgresql::VERSION = '2.4';
+$CGI::Session::Driver::postgresql::VERSION = '4.20';
 @CGI::Session::Driver::postgresql::ISA     = qw( CGI::Session::Driver::DBI );
 
 
 sub init {
     my $self = shift;
     my $ret = $self->SUPER::init(@_);
-    if (defined $self->{ColumnType}) {
-        no warnings "numeric";
-        return $ret if $self->{ColumnType} == PG_BYTEA || $self->{ColumnType} == PG_TEXT;
-        $self->{ColumnType} = lc(substr($self->{ColumnType},0,1)) eq 'b' ? PG_BYTEA : PG_TEXT;
-    } else {
-        $self->{ColumnType} = PG_TEXT;
-    }
+
+    # Translate external ColumnType into internal value. See POD for details.
+    $self->{PgColumnType} ||= (defined $self->{ColumnType} and (lc $self->{ColumnType} eq 'binary'))
+        ? PG_BYTEA
+        : PG_TEXT
+        ;
+
     return $ret;
 }
 
@@ -38,40 +38,46 @@ sub store {
     croak "store(): usage error" unless $sid && $datastr;
 
     my $dbh = $self->{Handle};
-    my $type = $self->{ColumnType};
+    my $type = $self->{PgColumnType};
 
     if ($type == PG_TEXT && $datastr =~ tr/\x00//) {
         croak "Unallowed characters used in session data. Please see CGI::Session::Driver::postgresql ".
             "for more information about null characters in text columns.";
     }
 
-    my $sth = $dbh->prepare("SELECT id FROM " . $self->table_name . " WHERE id=?");
-    unless ( defined $sth ) {
-        return $self->set_error( "store(): \$sth->prepare failed with message " . $dbh->errstr );
-    }
-
-    $sth->execute( $sid ) or return $self->set_error( "store(): \$sth->execute failed with message " . $dbh->errstr );
-    if ( $sth->fetchrow_array ) {
-        __ex_and_ret($dbh,"UPDATE " . $self->table_name . " SET a_session=? WHERE id=?",$datastr,$sid, $type)
-            or return $self->set_error( "store(): serialize to db failed " . $dbh->errstr );
-    } else {
-        __ex_and_ret($dbh,"INSERT INTO " . $self->table_name . " (a_session,id) VALUES(?, ?)",$datastr, $sid, $type)
-            or return $self->set_error( "store(): serialize to db failed " . $dbh->errstr );
-    }
-    return 1;
-}
-
-sub __ex_and_ret {
-    my ($dbh,$sql,$datastr,$sid,$type) = @_;
-    # fix rt #18183
-    local $@;
+    local $dbh->{RaiseError} = 1;
     eval {
-        my $sth = $dbh->prepare($sql) or return 0;
-        $sth->bind_param(1,$datastr,{ pg_type => $type }) or return 0;
-        $sth->bind_param(2,$sid) or return 0;
-        $sth->execute() or return 0;
+        # There is a race condition were two clients could run this code concurrently,
+        # and both end up trying to insert. That's why we check for "duplicate" below
+        my $sth = $dbh->prepare(
+             "INSERT INTO " . $self->table_name . " (a_session,id)  SELECT ?, ? 
+                WHERE NOT EXISTS (SELECT 1 FROM " . $self->table_name . " WHERE id=? LIMIT 1)");
+
+        $sth->bind_param(1,$datastr,{ pg_type => $type });
+        $sth->bind_param(2, $sid);
+        $sth->bind_param(3, $sid); # in the SELECT statement
+        my $rv = '';
+        eval { $rv = $sth->execute(); };
+        if ( $rv eq '0E0' or (defined $@ and $@ =~ m/duplicate/i) ) {
+            my $sth = $dbh->prepare("UPDATE " . $self->table_name . " SET a_session=? WHERE id=?");
+            $sth->bind_param(1,$datastr,{ pg_type => $type });
+            $sth->bind_param(2,$sid);
+            $sth->execute;
+        } 
+        else {
+            # Nothing. Our insert has already happened
+        }
     };
-    return ! $@;
+    if ($@) { 
+      return $self->set_error( "store(): failed with message: $@ " . $dbh->errstr );
+
+    } 
+    else {
+        return 1;
+
+    }
+
+
 }
 
 1;
